@@ -191,6 +191,9 @@ extern "C" {
 
 #if (defined(MRN_MARIADB_P) && MYSQL_VERSION_ID >= 100010)
 #  define MRN_HAVE_TDC_LOCK_TABLE_SHARE
+#  if MYSQL_VERSION_ID >= 100100
+#    define MRN_TABLE_SHARE_TDC_IS_POINTER
+#  endif
 #endif
 
 #ifdef MRN_MARIADB_P
@@ -597,6 +600,8 @@ private:
   void mkdir_p(const char *directory);
   ulonglong file_size(const char *path);
 
+  bool have_unique_index();
+
   void push_warning_unsupported_spatial_index_search(enum ha_rkey_function flag);
   void clear_cursor();
   void clear_cursor_geo();
@@ -634,6 +639,7 @@ private:
   void check_count_skip(key_part_map start_key_part_map,
                         key_part_map end_key_part_map, bool fulltext);
   bool is_grn_zero_column_value(grn_obj *column, grn_obj *value);
+  bool is_primary_key_field(Field *field) const;
   void check_fast_order_limit(grn_table_sort_key **sort_keys, int *n_sort_keys,
                               longlong *limit);
 
@@ -1095,6 +1101,7 @@ private:
   int wrapper_fill_indexes(THD *thd, KEY *key_info,
                            grn_obj **index_columns, uint n_keys);
   int wrapper_recreate_indexes(THD *thd);
+  int storage_recreate_indexes(THD *thd);
   int wrapper_repair(THD* thd, HA_CHECK_OPT* check_opt);
   int storage_repair(THD* thd, HA_CHECK_OPT* check_opt);
   bool wrapper_check_and_repair(THD *thd);
