@@ -1,7 +1,7 @@
 /* -*- c-basic-offset: 2 -*- */
 /*
   Copyright(C) 2011-2013 Kentoku SHIBA
-  Copyright(C) 2011-2014 Kouhei Sutou <kou@clear-code.com>
+  Copyright(C) 2011-2015 Kouhei Sutou <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -24,15 +24,35 @@
 #  include <sql_servers.h>
 #  include <sql_base.h>
 #endif
+#ifdef WITH_PARTITION_STORAGE_ENGINE
+#  include <partition_info.h>
+#endif
+#include <sql_plugin.h>
+
 #include "mrn_err.h"
 #include "mrn_table.hpp"
 #include "mrn_mysql_compat.h"
+#include "mrn_variables.hpp"
 #include <mrn_lock.hpp>
 
-#if MYSQL_VERSION_ID >= 50603 && !defined(MRN_MARIADB_P)
-#  define MRN_HA_RESOLVE_BY_NAME(name) ha_resolve_by_name(NULL, (name), TRUE)
+#ifdef MRN_MARIADB_P
+#  if MYSQL_VERSION_ID >= 100100
+#    define MRN_HA_RESOLVE_BY_NAME(name) ha_resolve_by_name(NULL, (name), TRUE)
+#  else
+#    define MRN_HA_RESOLVE_BY_NAME(name) ha_resolve_by_name(NULL, (name))
+#  endif
 #else
-#  define MRN_HA_RESOLVE_BY_NAME(name) ha_resolve_by_name(NULL, (name))
+#  if MYSQL_VERSION_ID >= 50603
+#    define MRN_HA_RESOLVE_BY_NAME(name) ha_resolve_by_name(NULL, (name), TRUE)
+#  else
+#    define MRN_HA_RESOLVE_BY_NAME(name) ha_resolve_by_name(NULL, (name))
+#  endif
+#endif
+
+#if MYSQL_VERSION_ID >= 50706 && !defined(MRN_MARIADB_P)
+#  define MRN_PLUGIN_DATA(plugin, type) plugin_data<type>(plugin)
+#else
+#  define MRN_PLUGIN_DATA(plugin, type) plugin_data(plugin, type)
 #endif
 
 #define LEX_STRING_IS_EMPTY(string)                                     \
@@ -47,19 +67,23 @@
 extern HASH *mrn_table_def_cache;
 #endif
 
-#ifdef WIN32
-#  ifdef MRN_TABLE_SHARE_HAVE_LOCK_SHARE
-extern PSI_mutex_key *mrn_table_share_lock_share;
-#  endif
-#  ifdef MRN_TABLE_SHARE_HAVE_LOCK_HA_DATA
-extern PSI_mutex_key *mrn_table_share_lock_ha_data;
-#  endif
-#endif
-
 #ifdef __cplusplus
 extern "C" {
 #endif
 
+#ifdef HAVE_PSI_INTERFACE
+#  ifdef WIN32
+#    ifdef MRN_TABLE_SHARE_HAVE_LOCK_SHARE
+extern PSI_mutex_key *mrn_table_share_lock_share;
+#    endif
+#    ifdef MRN_TABLE_SHARE_HAVE_LOCK_HA_DATA
+extern PSI_mutex_key *mrn_table_share_lock_ha_data;
+#    endif
+#  endif
+extern PSI_mutex_key mrn_share_mutex_key;
+extern PSI_mutex_key mrn_long_term_share_auto_inc_mutex_key;
+#endif
+
 extern HASH mrn_open_tables;
 extern mysql_mutex_t mrn_open_tables_mutex;
 extern HASH mrn_long_term_share;
@@ -69,8 +93,6 @@ extern char *mrn_default_wrapper_engine;
 extern handlerton *mrn_hton_ptr;
 extern HASH mrn_allocated_thds;
 extern mysql_mutex_t mrn_allocated_thds_mutex;
-extern PSI_mutex_key mrn_share_mutex_key;
-extern PSI_mutex_key mrn_long_term_share_auto_inc_mutex_key;
 
 static char *mrn_get_string_between_quote(const char *ptr)
 {
@@ -133,7 +155,7 @@ static char *mrn_get_string_between_quote(const char *ptr)
     DBUG_RETURN(NULL);
 
   size_t length = end_ptr - start_ptr;
-  char *extracted_string = (char *)my_malloc(length + 1, MYF(MY_WME));
+  char *extracted_string = (char *)mrn_my_malloc(length + 1, MYF(MY_WME));
   if (esc_flg) {
     size_t extracted_index = 0;
     const char *current_ptr = start_ptr;
@@ -370,9 +392,9 @@ int mrn_parse_table_param(MRN_SHARE *share, TABLE *table)
     }
 
     {
-      params_string = my_strndup(params_string_value,
-                                 params_string_length,
-                                 MYF(MY_WME));
+      params_string = mrn_my_strndup(params_string_value,
+                                     params_string_length,
+                                     MYF(MY_WME));
       if (!params_string) {
         error = HA_ERR_OUT_OF_MEM;
         goto error;
@@ -435,9 +457,9 @@ int mrn_parse_table_param(MRN_SHARE *share, TABLE *table)
   {
     share->engine_length = strlen(mrn_default_wrapper_engine);
     if (
-      !(share->engine = my_strndup(mrn_default_wrapper_engine,
-                                   share->engine_length,
-                                   MYF(MY_WME)))
+      !(share->engine = mrn_my_strndup(mrn_default_wrapper_engine,
+                                       share->engine_length,
+                                       MYF(MY_WME)))
     ) {
       error = HA_ERR_OUT_OF_MEM;
       goto error;
@@ -469,7 +491,7 @@ int mrn_parse_table_param(MRN_SHARE *share, TABLE *table)
         error = ER_UNKNOWN_STORAGE_ENGINE;
         goto error;
       }
-      share->hton = plugin_data(share->plugin, handlerton *);
+      share->hton = MRN_PLUGIN_DATA(share->plugin, handlerton *);
       share->wrapper_mode = TRUE;
     }
   }
@@ -505,7 +527,7 @@ int mrn_add_index_param(MRN_SHARE *share, KEY *key_info, int i)
       my_free(share->key_parser[i]);
     }
     if (
-      !(share->key_parser[i] = my_strdup(mrn_default_parser, MYF(MY_WME)))
+      !(share->key_parser[i] = mrn_my_strdup(mrn_default_parser, MYF(MY_WME)))
     ) {
       error = HA_ERR_OUT_OF_MEM;
       goto error;
@@ -515,9 +537,9 @@ int mrn_add_index_param(MRN_SHARE *share, KEY *key_info, int i)
   }
   DBUG_PRINT("info", ("mroonga create comment string"));
   if (
-    !(param_string = my_strndup(key_info->comment.str,
-                                key_info->comment.length,
-                                MYF(MY_WME)))
+    !(param_string = mrn_my_strndup(key_info->comment.str,
+                                    key_info->comment.length,
+                                    MYF(MY_WME)))
   ) {
     error = HA_ERR_OUT_OF_MEM;
     goto error_alloc_param_string;
@@ -567,7 +589,7 @@ int mrn_add_index_param(MRN_SHARE *share, KEY *key_info, int i)
 #endif
   if (!share->key_parser[i]) {
     if (
-      !(share->key_parser[i] = my_strdup(mrn_default_parser, MYF(MY_WME)))
+      !(share->key_parser[i] = mrn_my_strdup(mrn_default_parser, MYF(MY_WME)))
     ) {
       error = HA_ERR_OUT_OF_MEM;
       goto error;
@@ -628,9 +650,9 @@ int mrn_add_column_param(MRN_SHARE *share, Field *field, int i)
 
   DBUG_PRINT("info", ("mroonga create comment string"));
   if (
-    !(param_string = my_strndup(field->comment.str,
-                                field->comment.length,
-                                MYF(MY_WME)))
+    !(param_string = mrn_my_strndup(field->comment.str,
+                                    field->comment.length,
+                                    MYF(MY_WME)))
   ) {
     error = HA_ERR_OUT_OF_MEM;
     goto error_alloc_param_string;
@@ -766,7 +788,7 @@ MRN_LONG_TERM_SHARE *mrn_get_long_term_share(const char *table_name,
                    table_name_length)))
   {
     if (!(long_term_share = (MRN_LONG_TERM_SHARE *)
-      my_multi_malloc(MYF(MY_WME | MY_ZEROFILL),
+      mrn_my_multi_malloc(MYF(MY_WME | MY_ZEROFILL),
         &long_term_share, sizeof(*long_term_share),
         &tmp_name, table_name_length + 1,
         NullS))
@@ -815,7 +837,7 @@ MRN_SHARE *mrn_get_share(const char *table_name, TABLE *table, int *error)
     (uchar*) table_name, length)))
   {
     if (!(share = (MRN_SHARE *)
-      my_multi_malloc(MYF(MY_WME | MY_ZEROFILL),
+      mrn_my_multi_malloc(MYF(MY_WME | MY_ZEROFILL),
         &share, sizeof(*share),
         &tmp_name, length + 1,
         &index_table, sizeof(char *) * table->s->keys,
@@ -845,7 +867,7 @@ MRN_SHARE *mrn_get_share(const char *table_name, TABLE *table, int *error)
     share->col_flags_length = col_flags_length;
     share->col_type = col_type;
     share->col_type_length = col_type_length;
-    strmov(share->table_name, table_name);
+    mrn_my_stpmov(share->table_name, table_name);
     share->table_share = table->s;
 
     if (
@@ -916,7 +938,7 @@ MRN_SHARE *mrn_get_share(const char *table_name, TABLE *table, int *error)
     }
 
     if (mysql_mutex_init(mrn_share_mutex_key,
-                         &share->mutex,
+                         &share->record_mutex,
                          MY_MUTEX_INIT_FAST) != 0)
     {
       *error = HA_ERR_OUT_OF_MEM;
@@ -939,7 +961,7 @@ MRN_SHARE *mrn_get_share(const char *table_name, TABLE *table, int *error)
 
 error_hash_insert:
 error_get_long_term_share:
-  mysql_mutex_destroy(&share->mutex);
+  mysql_mutex_destroy(&share->record_mutex);
 error_init_mutex:
 error_parse_table_param:
   mrn_free_share_alloc(share);
@@ -959,7 +981,7 @@ int mrn_free_share(MRN_SHARE *share)
       plugin_unlock(NULL, share->plugin);
     mrn_free_share_alloc(share);
     thr_lock_delete(&share->lock);
-    mysql_mutex_destroy(&share->mutex);
+    mysql_mutex_destroy(&share->record_mutex);
     if (share->wrapper_mode) {
 #ifdef MRN_TABLE_SHARE_HAVE_LOCK_SHARE
       mysql_mutex_destroy(&(share->wrap_table_share->LOCK_share));
@@ -1030,9 +1052,9 @@ TABLE_SHARE *mrn_create_tmp_table_share(TABLE_LIST *table_list, const char *path
   }
   share->tmp_table = INTERNAL_TMP_TABLE; // TODO: is this right?
   share->path.str = (char *) path;
-  share->path.length = strlen(path);
-  share->normalized_path.str = share->path.str;
-  share->normalized_path.length = share->path.length;
+  share->path.length = strlen(share->path.str);
+  share->normalized_path.str = mrn_my_strdup(path, MYF(MY_WME));
+  share->normalized_path.length = strlen(share->normalized_path.str);
   if (open_table_def(thd, share, GTS_TABLE))
   {
     *error = ER_CANT_OPEN_FILE;
@@ -1044,7 +1066,9 @@ TABLE_SHARE *mrn_create_tmp_table_share(TABLE_LIST *table_list, const char *path
 void mrn_free_tmp_table_share(TABLE_SHARE *tmp_table_share)
 {
   MRN_DBUG_ENTER_FUNCTION();
+  char *normalized_path = tmp_table_share->normalized_path.str;
   free_table_share(tmp_table_share);
+  my_free(normalized_path);
   DBUG_VOID_RETURN;
 }
 
@@ -1056,7 +1080,7 @@ KEY *mrn_create_key_info_for_table(MRN_SHARE *share, TABLE *table, int *error)
   if (share->wrap_keys)
   {
     if (!(wrap_key_info = (KEY *)
-      my_multi_malloc(MYF(MY_WME | MY_ZEROFILL),
+      mrn_my_multi_malloc(MYF(MY_WME | MY_ZEROFILL),
         &wrap_key_info, sizeof(*wrap_key_info) * share->wrap_keys,
         NullS))
     ) {
